#!/usr/bin/perl
###############################################################################
#    Copyright (C) 2002-2204 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: afick_postinstall.pl 2549 2012-07-11 08:09:47Z gerbier $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this script is to be called at end of install
# it is in perl to allow it to work on windows

use strict;
use warnings;

use Getopt::Long;      # option analysis
use File::Basename;    # dirname
use File::Spec;        # rel2abs
use English qw(-no_match_vars);

my $Version = '0.5';

#############################################################
# just display program version
sub version($) {
	my $version = shift @_;
	print "\n";
	print
"afick_postinstall : another file integrity checker configurator\nversion $version\n";
	return;
}
#############################################################
# usage
# print some help
sub usage($) {
	my $version = shift @_;
	print <<"EOHELP";

Usage: $PROGRAM_NAME [options]

afick_postinstall options (control afickonfig)  : 
 -c|--config_file file        configname of config file to use
 -h|--help                    show this help page
 -V|--version                 show afickonfig version
 -v|--verbose		     for debugging


Disclaimer:
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 

Copyright (c) 2006 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
EOHELP
	return;
}
#############################################################

# afick library
# it is used on post-install, so all should be ok
my $dirname = File::Spec->rel2abs( dirname($PROGRAM_NAME) );
require $dirname . '/afick-common.pl';

$OUTPUT_AUTOFLUSH = 1;

# arg line
my $opt_configfile;
my $opt_help;
my $opt_version;
my $opt_verbose;
Getopt::Long::Configure('no_ignore_case');
if (
	!GetOptions(
		'config_file|c=s' => \$opt_configfile,
		'help|h'          => \$opt_help,
		'verbose|v'       => \$opt_verbose,
		'version|V'       => \$opt_version,
	)
  )
{
	usage($Version);
	die "abort : incorrect option\n";

}
if ($opt_help) {

	# -h : help
	usage($Version);
	exit;
}
elsif ($opt_version) {

	# -V : version
	version($Version);
	exit;
}

if ( defined $opt_verbose ) {
	Afick::Msg->set_verbose($opt_verbose);
}

if ( !$opt_configfile ) {
	$opt_configfile = get_default_config();
}
Afick::Msg->debug("configuration file is $opt_configfile");

# first : create empty history file if a new install (not an upgrade)
# to avoid clean_config to remove directive

# get history file name from configuration file
my %macros;
my %alias = get_default_alias();
my %directive;
my %rules;
my %onlydir;

get_configuration( $opt_configfile, \%macros, \%alias, \%directive, \%rules,
	\%onlydir );
my $history = $directive{'history'};

# create it if necessary
touch($history) if ($history);

# second : append local config file to config file if it exists
# to avoid duplicate copying, we read first until the mark then write
# (just append at end may append many times the same lines)
# be carefull : this is also defined in afick_preinstall !
my $mark         = 'put your local config below';
my $local_config = $opt_configfile . '.local';

# read current config to mark and store
Afick::Msg->debug('modify configuration file');
## no critic (RequireBriefOpen)
my $fh_config;
if ( !open $fh_config, '<', $opt_configfile ) {
	die "can not read $opt_configfile : $ERRNO\n";
}
my @config;
LOOP: while (<$fh_config>) {
	push @config, $_;
	last LOOP if (m/$mark/);
}
close $fh_config
  or Afick::Msg->warning("can not close $opt_configfile : $ERRNO");

Afick::Msg->debug('rewrite config');

# rewrite it
if ( !open $fh_config, '>', $opt_configfile ) {
	die "can not write to $opt_configfile : $ERRNO\n";
}
foreach my $line (@config) {
	print {$fh_config} $line;
}

# add local config
Afick::Msg->debug('add local config');
if ( -s $local_config ) {
	my $fh_local;
	if ( open $fh_local, '<', $local_config ) {

		while (<$fh_local>) {
			print {$fh_config} $_;
			Afick::Msg->debug("copy line $_");
		}
		close $fh_local
		  or Afick::Msg->warning("can not close $local_config : $ERRNO");
	}
	else {
		Afick::Msg->warning("can not open $local_config : $ERRNO");
	}
	close $fh_config
	  or Afick::Msg->warning("can not close $opt_configfile : $ERRNO");

	# remove local config
	unlink $local_config;
}

# third : clean configuration
Afick::Msg->debug('clean configuration');
Afick::Msg->debug("path = $dirname");
system $EXECUTABLE_NAME, $dirname . '/afickonfig.pl', '-c', $opt_configfile,
  '--clean_config';
system $EXECUTABLE_NAME, $dirname . '/afickonfig.pl', '-c', $opt_configfile,
  '--addpath', '--addlib';
